/*
 *	I2C External EEPROM example using SimpleC
 *	This code will write 10 bytes to the EEPROM and 
 *	then read them back and display binary value on LEDs.
 * Connections:
 * Pin 8(B0) - LED Anode - LED Cathode - 470ohm - Ground
 * Pin 9(B1) - LED Anode - LED Cathode - 470ohm - Ground
 * Pin 10(B2) - LED Anode - LED Cathode - 470ohm - Ground
 * Pin 11(B3) - LED Anode - LED Cathode - 470ohm - Ground
 *
 * Pin 3(C4) - 24LC02B SDA - 10k pullup - 5v
 * Pin 4(C3) - 24LC02B SCL - 10k pullup - 5v
 * 24LC02B Vss - Ground
 * 24LC02B Vdd - 5v
 */

#define _LEGACY_HEADERS //Added for compiler versions 9.81+ 

#include <htc.h>
#include <stdio.h>
__CONFIG (HS & WDTDIS & MCLREN & UNPROTECT & LVPDIS & IESODIS & FCMDIS);


//*******************************************************
// Initialize Microcontroller Setup
//*******************************************************

void init_micro(void)
{
	ANSEL = 0; 		//Analog ports set to digital		
	ANSELH = 0;
	CM1CON0 = 0;	//Comparators Shut off
	CM2CON0 = 0;
	INTCON=0;	// purpose of disabling the interrupts
}

/********************************************************
* msecbase - 1 msec pause routine						*
* The External oscillator is set to 16 Mhz and the		*
* internal instruction clock is 1/4 of the oscillator.	*
* This makes the internal instruction clock 4 Mhz or	*
* 0.25 usec per clock pulse.							*
* Using the 1:4 prescaler on the clock input to Timer0	* 
* slows the Timer0 count increment to 1 count/1 usec.	*
* Therefore 250 counts of the Timer0 would make a one	*
* millisecond delay (250 * 4 usec). But there are other	*
* instructions in the delay loop so using the MPLAB 	*
* stopwatch we find that we need Timer0 to overflow at 	*
* 221 clock ticks. Preset Timer0 to 25 (19 hex) to make	*
* Timer0 overflow at 243 clock ticks (256-25 = 221).	*
* This results in a 1.056 millisecond delay which is 	*
* as close t0 1.0 msec as we can get.					*
*********************************************************/
void msecbase(void)
{
	OPTION = 0b00000011;		//Set prescaler to TMRO 1:4
	TMR0 = 0xD;					//Preset TMRO to overflow on 250 counts
	while(!T0IF);				//Stay until TMRO overflow flag equals 1
	T0IF = 0;					//Clear the TMR0 overflow flag
}


//**********************************************************
//pause - millisecond delay routine 1 to 65,535 milliseconds
//**********************************************************


void pause( unsigned long usvalue )

{
	unsigned long x;		
	for (x=1; x<=(usvalue); x++)	//Loop through a delay equal to usvalue
		{						// in milliseconds.
		msecbase();				//Jump to millisec delay routine
		}
}


/******************************************************************************************/
// Hardware I2C single master routines for PIC16F886
// for HI-TECH PIC C COMPILER.
//
// i2c_init  - initialize I2C functions
// i2c_start - issue Start condition
// i2c_repStart- issue Repeated Start condition
// i2c_stop  - issue Stop condition
// i2c_read(x) - receive unsigned char - x=0, don't acknowledge - x=1, acknowledge
// i2c_write - write unsigned char - returns ACK
//
//
//******************************************************************************************

void i2c_init()
{
 TRISC3=1;        // set SCL and SDA pins as inputs
 TRISC4=1;

 SSPCON = 0x38;   // set I2C master mode, CKP = 1, SSPEN = 1
 SSPCON2 = 0x00;

// Formula for SSPADD register value: SSPADD = [(Fosc/4)/BitRate] - 1
 SSPADD = 0x27;     // 100k at 16Mhz clock
			// [(16Mhz/4)/100khz]-1 = 39 or 0x27 hex

// CKE=1;     	// use I2C levels      worked also with '0'
// SMP=1;     	// disable slew rate control  worked also with '0'

 SSPIF=0;      	// clear SSPIF interrupt flag
 BCLIF=0;      	// clear bus collision flag
}

//******************************************************************************************

void i2c_waitForIdle()
{
 while (( SSPCON2 & 0x1F ) | RW ) {}; // wait for idle and not writing
}

//******************************************************************************************

void i2c_start()
{
 i2c_waitForIdle();
 SEN=1;
}

//******************************************************************************************

void i2c_repStart()
{
 i2c_waitForIdle();
 RSEN=1;
}

//******************************************************************************************

void i2c_stop()
{
 i2c_waitForIdle();
 PEN=1;
}

//******************************************************************************************

int i2c_read( unsigned char ack )
{
 unsigned char i2cReadData;

 i2c_waitForIdle();

 RCEN=1;

 i2c_waitForIdle();

 i2cReadData = SSPBUF;

 i2c_waitForIdle();

 if ( ack )
  {
  ACKDT=0;
  }
 else
  {
  ACKDT=1;
  }
  ACKEN=1;               // send acknowledge sequence

 return( i2cReadData );
}

//******************************************************************************************

unsigned char i2c_write( unsigned char i2cWriteData )
{
 i2c_waitForIdle();
 SSPBUF = i2cWriteData;

 return ( ! ACKSTAT  ); // function returns '1' if transmission is acknowledged
}

// *********************************************************************************
// i2c_eeout function to write to an 8-bit address Microchip EEPROM part (i.e. 24LC256)
// *********************************************************************************


void I2C_EEOUT(unsigned char address, unsigned char data)
 {
   i2c_start();
   i2c_write(0xa0);
   i2c_write(address);
   i2c_write(data);
   i2c_stop();
   pause(11);
}


// *********************************************************************************
// i2c_eein function to read from an 8-bit address Microchip EEPROM part (i.e. 24LC256)
// *********************************************************************************

unsigned char I2C_EEIN(unsigned char address)
{
   unsigned char data;

   i2c_start();
   i2c_write(0xa0);
   i2c_write(address);
   i2c_repStart();
   i2c_write(0xa1);
   data=i2c_read(0);
   i2c_stop();
   return(data);
}

// ********************************************************************
//  Main Program Loop
// ********************************************************************

void
main(void)
{
init_micro();	// Initialize I/O
i2c_init();    	// Initialize i2c 
PORTB=0x00;    	// All LEDs off
TRISB=0x00;    	// All PORTB outputs

unsigned char cnt=0;		// Create variable and initialize to zero
 
 while(cnt<=10)   /* write 10 numbers to eeprom */
  {        
  PORTB=cnt;         		// The cnt value will display on the LEDs
  I2C_EEOUT(cnt,10-cnt);   	// Store 9 at address 1, 8 at address 2, ...
  cnt=cnt+1;				// Increment cnt variable
  pause(255);				// Delay 255 milliseconds
  }

 cnt=0;

 while(cnt<=10)
  {
 PORTB = I2C_EEIN(cnt); // Display data read from EEPROM on LEDs in binary 
 pause (1000);			// Delay for 1 second to read LEDs
 cnt=cnt + 1;			// Increment cnt variable
  }
}
